'' =================================================================================================
''
''   File....... jm_dyna_report_v2.spin
''   Purpose.... * reads entire contents of servo in one packet
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2013 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 15 SEP 2013 
''
'' =================================================================================================


con  { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = ((_clkmode - xtal1) >> 6) * _xinfreq               ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con  { io connections for EFX-TEK HC-8+ }

  RX1     = 31                                                  ' programming / debug port
  TX1     = 30
  
  SDA     = 29                                                  ' eeprom
  SCL     = 28

  OUT_EN  = 27                                                  ' outputs enable pin (low)     
  
  SIO     = 26                                                  ' TTL serial io
  
  RX2     = 23                                                  ' RS-485 port
  RX2_EN  = 22
  TX2_EN  = 21
  TX2     = 20

  R_LED   = 18                                                  ' bi-color LED
  G_LED   = 17

  OUT7    = 15                                                  ' outputs
  OUT6    = 14   
  OUT5    = 13 
  OUT4    = 12
  OUT3    = 11
  OUT2    = 10   
  OUT1    =  9 
  OUT0    =  8
          
  OPT_BR  =  7                                                  ' options swithes
  OPT_A1  =  6   
  OPT_A0  =  5
  OPT_SM  =  4

  DMX_A8  =  3                                                  ' high-bit off DMX address

  DO_165  =  2                                                  ' control lines for 74x165
  CLK_165 =  1 
  LD_165  =  0


  DYNA    =  OPT_BR                                             ' dynamixel connection
  

con  { pst constants }

   #1, HOME, GOTOXY, #8, BKSP, TAB, LF, CLREOL, CLRDN, CR       ' PST formmatting control
  #14, GOTOX, GOTOY, CLS


con

  #1, PAN, TILT

  X1 = 19
  X2 = 29
  

obj
                                     
  dy   : "jm_dynamixel"
  term : "fullduplexserial64"
 

var

  byte  regbuf[64]                                      ' copy of dynamixel registers


dat

  Report                byte    "                   Servo #1  Servo #2", CR
                        byte    "                   --------  --------", CR
                        byte    "Ping.............. ", CR
                        byte    "Errors............ ", CR
                        byte    "Model............. ", CR
                        byte    "Version........... ", CR
                        byte    CR
                        byte    "Volts............. ", CR
                        byte    "Temperature....... ", CR
                        byte    CR
                        byte    "CW Limit.......... ", CR
                        byte    "CW Comp Margin.... ", CR
                        byte    "CW Comp Slope..... ", CR
                        byte    CR
                        byte    "CCW Limit......... ", CR
                        byte    "CCW Comp Margin... ", CR
                        byte    "CCW Comp Slope.... ", CR
                        byte    CR
                        byte    "Punch............. ", CR
                        byte    "Max Torque........ ", CR
                        byte    "Torque Enabled.... ", CR
                        byte    CR
                        byte    "Position.......... ", CR
                        byte    CR
                        byte    0                      


pub main | t, check, value

  dy.start(DYNA, 1_000_000)
  term.start(RX1, TX1, %0000, 115_200)
  pause(10)

  term.rxflush  { press key when ready }
  term.rx

  term.tx(CLS)
  term.str(@Report)

  ' remove torque from dynamixels
  ' -- allow for head positioning

  dy.tx_clear
  dy.tx_putbyte( 7,  PAN)
  dy.tx_putbyte( 8,    0)
  dy.tx_putbyte( 9, TILT)
  dy.tx_putbyte(10,    0)

  dy.sync_write(2, dy#P_TORQUE_ENABLE, 1)
  

  t := cnt
  repeat
    { pan servo }
    waitcnt(t += constant(100 * MS_001))                        ' pad for packet
    bytefill(@regbuf, 0, 64)                                    ' clear buffer
    check := dy.read_data(PAN, 0, @regbuf, 50)                  ' pull entire dynamixel ram
    
    cursor_xy(X1, 02)
    if (check <> PAN)
      term.str(string("      No"))   
      clear_section(PAN)
     
    else
      term.str(string("     Yes"))

      cursor_xy(X1, 03)          
      term.bin(dy.rx_error, 8)   
       
      value := dy.get_byte(@regbuf, dy#P_MODEL_NUMBER_L)
      rjdec_xyw(value, X1, 04, 8)
       
      value := dy.get_byte(@regbuf, dy#P_VERSION)
      rjdec_xyw(value, X1, 05, 8)
       
      value := dy.get_byte(@regbuf, dy#P_PRESENT_VOLTAGE)
      rjdec_xyw(value/10, X1, 07, 6)
      term.tx(".")
      term.tx(value//10 + "0")
       
      value := dy.get_byte(@regbuf, dy#P_PRESENT_TEMPERATURE)
      rjdec_xyw(value, X1, 08, 8)

      value := dy.get_word(@regbuf, dy#P_CW_ANGLE_LIMIT_L)  
      rjdec_xyw(value, X1, 10, 8) 

      value := dy.get_byte(@regbuf, dy#P_CW_COMPLIANCE_MARGIN)  
      rjdec_xyw(value, X1, 11, 8)

      value := dy.get_byte(@regbuf, dy#P_CW_COMPLIANCE_SLOPE)  
      rjdec_xyw(value, X1, 12, 8)      

      value := dy.get_word(@regbuf, dy#P_CCW_ANGLE_LIMIT_L)  
      rjdec_xyw(value, X1, 14, 8) 

      value := dy.get_byte(@regbuf, dy#P_CCW_COMPLIANCE_MARGIN)  
      rjdec_xyw(value, X1, 15, 8)

      value := dy.get_byte(@regbuf, dy#P_CCW_COMPLIANCE_SLOPE)  
      rjdec_xyw(value, X1, 16, 8)

      value := dy.get_word(@regbuf, dy#P_PUNCH_L)  
      rjdec_xyw(value, X1, 18, 8)

      value := dy.get_word(@regbuf, dy#P_MAX_TORQUE_L)  
      rjdec_xyw(value, X1, 19, 8)

      value := dy.get_byte(@regbuf, dy#P_TORQUE_ENABLE)  
      rjdec_xyw(value, X1, 20, 8)      
        
      value := dy.get_word(@regbuf, dy#P_PRESENT_POSITION_L)  
      rjdec_xyw(value, X1, 22, 8)


    { tilt servo }
    waitcnt(t += constant(100 * MS_001))                        ' pad for packet
    bytefill(@regbuf, 0, 64)                                    ' clear buffer
    check := dy.read_data(TILT, 0, @regbuf, 50)                 ' pull entire dynamixel ram
    
    cursor_xy(X2, 02)
    if (check <> TILT)
      term.str(string("      No"))   
      clear_section(TILT)
     
    else
      term.str(string("     Yes"))

      cursor_xy(X2, 03)          
      term.bin(dy.rx_error, 8)   
       
      value := dy.get_byte(@regbuf, dy#P_MODEL_NUMBER_L)
      rjdec_xyw(value, X2, 04, 8)
       
      value := dy.get_byte(@regbuf, dy#P_VERSION)
      rjdec_xyw(value, X2, 05, 8)
       
      value := dy.get_byte(@regbuf, dy#P_PRESENT_VOLTAGE)
      rjdec_xyw(value/10, X2, 07, 6)
      term.tx(".")
      term.tx(value//10 + "0")
       
      value := dy.get_byte(@regbuf, dy#P_PRESENT_TEMPERATURE)
      rjdec_xyw(value, X2, 08, 8)

      value := dy.get_word(@regbuf, dy#P_CW_ANGLE_LIMIT_L)  
      rjdec_xyw(value, X2, 10, 8) 

      value := dy.get_byte(@regbuf, dy#P_CW_COMPLIANCE_MARGIN)  
      rjdec_xyw(value, X2, 11, 8)

      value := dy.get_byte(@regbuf, dy#P_CW_COMPLIANCE_SLOPE)  
      rjdec_xyw(value, X2, 12, 8)      

      value := dy.get_word(@regbuf, dy#P_CCW_ANGLE_LIMIT_L)  
      rjdec_xyw(value, X2, 14, 8) 

      value := dy.get_byte(@regbuf, dy#P_CCW_COMPLIANCE_MARGIN)  
      rjdec_xyw(value, X2, 15, 8)

      value := dy.get_byte(@regbuf, dy#P_CCW_COMPLIANCE_SLOPE)  
      rjdec_xyw(value, X2, 16, 8)

      value := dy.get_word(@regbuf, dy#P_PUNCH_L)  
      rjdec_xyw(value, X2, 18, 8)

      value := dy.get_word(@regbuf, dy#P_MAX_TORQUE_L)  
      rjdec_xyw(value, X2, 19, 8)

      value := dy.get_byte(@regbuf, dy#P_TORQUE_ENABLE)  
      rjdec_xyw(value, X2, 20, 8)      
        
      value := dy.get_word(@regbuf, dy#P_PRESENT_POSITION_L)  
      rjdec_xyw(value, X2, 22, 8)
            

pub clear_section(s) | line

  repeat line from 3 to 22
    cursor_xy(X1 + (10 * (s - 1)), line)
    term.str(string("        "))

   
pub cursor_xy(x, y)

  term.tx(GOTOXY)
  term.tx(x)
  term.tx(y)
  

pub rjdec_xyw(value, x, y, w)

  cursor_xy(x, y)
  term.rjdec(value, w, " ")


con

  { --------------- }
  {                 }
  {   B A S I C S   }
  {                 }
  { --------------- }


pub pause(ms) | t

'' Delay program in milliseconds

  if (ms < 1)                                                   ' delay must be > 0
    return
  else
    t := cnt - 1776                                             ' sync with system counter
    repeat ms                                                   ' run delay
      waitcnt(t += MS_001)
    

pub high(pin)

'' Makes pin output and high

  outa[pin] := 1
  dira[pin] := 1


pub low(pin)

'' Makes pin output and low

  outa[pin] := 0
  dira[pin] := 1


pub toggle(pin)

'' Toggles pin state

  !outa[pin]
  dira[pin] := 1


pub input(pin)

'' Makes pin input and returns current state

  dira[pin] := 0

  return ina[pin]
  

dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}